package main

import (
	"fmt"
	"os"
	"strconv"

	"github.com/charmbracelet/lipgloss"
	"github.com/mattn/go-tty"
)

// TODO consider local-only help command for renaming and adding emails to account

type colorScheme struct {
	Header   func(string) string
	Subtitle func(string) string
	Prompt   func(string) string
	Email    func(string) string
	Option   func(string) string
}

func newColorScheme() colorScheme {
	s2r := func(s lipgloss.Style) func(string) string {
		return s.Render
	}
	c := func(s string) lipgloss.Color {
		return lipgloss.Color(s)
	}
	s := lipgloss.NewStyle
	return colorScheme{
		Header:   s2r(s().Bold(true).Foreground(c("#E0B0FF"))),
		Subtitle: s2r(s().Italic(true).Foreground(c("gray"))),
		Email:    s2r(s().Bold(true).Underline(true)),
		Prompt:   s2r(s().Bold(true).Foreground(c("#00752d"))),
		Option:   s2r(s().Bold(true).Foreground(c("#38747a"))),
	}
}

func stringPrompt(cs colorScheme, tty *tty.TTY, prompt string) (string, error) {
	fmt.Println("")
	fmt.Println(cs.Prompt(prompt))
	fmt.Println(cs.Subtitle("(type your answer below and press enter to submit)"))
	s, err := tty.ReadString()
	if err != nil {
		return "", fmt.Errorf("couldn't collect input: %w", err)
	}

	return s, nil
}

func numberPrompt(cs colorScheme, tty *tty.TTY, prompt string, opts []string) (int, error) {
	fmt.Println()
	fmt.Println(cs.Prompt(prompt))
	fmt.Println(cs.Subtitle("(pick an option using the corresponding number)"))

	chosen := -1
	for chosen < 0 {
		fmt.Println()
		for ix, o := range opts {
			fmt.Printf("%s: %s\n", cs.Option(fmt.Sprintf("%d", ix+1)), o)
		}
		r, err := tty.ReadRune()
		if err != nil {
			return -1, fmt.Errorf("could not collect answer for '%s': %w", prompt, err)
		}
		c, err := strconv.Atoi(string(r))
		if err != nil {
			fmt.Println()
			fmt.Printf("I could not understand '%s'. Try again, please.\n", string(r))
			continue
		}
		if c > len(opts) || c == 0 {
			fmt.Println()
			fmt.Printf("%s is not an option. Try again, please.\n", string(r))
			continue
		}
		chosen = c - 1
	}

	fmt.Println("")

	return chosen, nil
}

func _main() error {
	cs := newColorScheme()
	fmt.Println(cs.Header("Hi, you have reached the tilde town help desk."))
	fmt.Println()
	fmt.Println("Please check out the options below.")
	fmt.Printf("If none of them apply to your case, you can email %s. \n", cs.Email("root@tilde.town"))
	tty, err := tty.Open()
	if err != nil {
		return fmt.Errorf("could not open tty: %w", err)
	}
	defer tty.Close()

	options := []string{
		"I need to request that a new SSH key be added to my account.",
		"I have a code from my e-mail to redeem for a new SSH key",
		"I just want out of here",
	}
	c, err := numberPrompt(cs, tty, "What do you need help with?", options)

	switch c {
	case 0:
		return collectEmail(cs, tty)
	case 1:
		return redeemCode(tty)
	case 2:
		fmt.Println()
		fmt.Println(cs.Header("bye~"))
		return nil
	}

	return nil
}

func collectEmail(cs colorScheme, tty *tty.TTY) error {
	fmt.Println(cs.Header("We can send a reset code to an email associated with your town account."))
	email, err := stringPrompt(cs, tty, "email to send reset code to?")
	if err != nil {
		return err
	}

	fmt.Println(email)
	// TODO confirm email
	// TODO generate reset code
	// TODO send email
	// TODO report success
	return nil
}

func redeemCode(tty *tty.TTY) error {
	// TODO deserialize
	// TODO verify code
	// TODO accept key
	// TODO verify key
	// TODO add key to .authorized_keys2
	// TODO mark used
	return nil
}

func main() {
	err := _main()
	if err != nil {
		fmt.Printf("sorry, something went wrong: %s\n", err.Error())
		fmt.Println("Please let an admin know by emailing a copy of this error to root@tilde.town")
		os.Exit(1)
	}
}
